/***************************************************************************
			kledbutton.cpp - KLed widget for kommander
                             -------------------
		begin			: 11-5-2005
		copyright		: (C) Pedro Lopez-Cabanillas
		email			: plcl@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* KDE INCLUDES */
#include <klocale.h>
#include <kled.h>

/* QT INCLUDES */
#include <qstring.h>
#include <qwidget.h>
#include <qstringlist.h>
#include <qevent.h>

/* OTHER INCLUDES */
#include <kommanderwidget.h>
#include "kledbutton.h"
#include "dcopspecials.h"

KLedButton::StateMap KLedButton::m_stateMap;

KLedButton::KLedButton(QWidget *a_parent, const char *a_name)
	: KLed(a_parent, a_name), KommanderWidget(this)
{
	QStringList states;
	states << "unchecked";
	states << "checked";
	setStates(states);
	
	QStringList displayStates;
	displayStates << "checked";
	displayStates << "unchecked";
	setDisplayStates(displayStates);
	
	if (m_stateMap.empty()) {
	    m_stateMap[KLed::Off] = "unchecked";
	    m_stateMap[KLed::On] = "checked";
	}
}

KLedButton::~KLedButton()
{
}

QString KLedButton::currentState() const
{
	return m_stateMap[state()];
}

bool KLedButton::isKommanderWidget() const
{
	return TRUE;
}

QStringList KLedButton::associatedText() const
{
	return KommanderWidget::associatedText();
}

void KLedButton::setAssociatedText(QStringList a_at)
{
	KommanderWidget::setAssociatedText(a_at);
}

void KLedButton::populate()
{
	QString txt = KommanderWidget::evalAssociatedText( populationText() );
	setWidgetText( txt );
}

QString KLedButton::populationText() const
{
	return KommanderWidget::populationText();
}

void KLedButton::setPopulationText(QString a_text)
{
	KommanderWidget::setPopulationText(a_text);
}

void KLedButton::setWidgetText(const QString &a_text)
{
	QString text = a_text.lower().stripWhiteSpace();
	StateMap::const_iterator i;
	for ( i = m_stateMap.begin(); i != m_stateMap.end(); ++i) {
	    if (text == i->second) {
		setState(i->first);
		emit widgetTextChanged(text);
	    } 
	}
}

QString KLedButton::widgetText() const
{
	return m_stateMap[state()];
}

void KLedButton::showEvent( QShowEvent *e )
{
	KLed::showEvent( e );
	emit widgetOpened();
}

void KLedButton::mousePressEvent( QMouseEvent *e )
{
	if (e->button() == LeftButton) {
	    toggle();
	    emit stateChanged( state() );
	}
}

QSize KLedButton::sizeHint() const
{
	return QSize(16, 16);
}

QSize KLedButton::minimumSizeHint() const
{
	return QSize(16, 16);
}

QSizePolicy KLedButton::sizePolicy () const
{
	return QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
}

bool KLedButton::isFunctionSupported(int f)
{
	return f == DCOP::text || f == DCOP::setText || f == DCOP::checked || f == DCOP::setChecked;
}

QString KLedButton::handleDCOP(int function, const QStringList& args)
{
	switch (function) {
	case DCOP::text:
	    return widgetText();
	case DCOP::setText:
	    setWidgetText(args[0]);
	    break;
	case DCOP::checked:
	    return (state() == KLed::On ? "true" : "false" );
	case DCOP::setChecked:
	    setState(args[0] == "true" ? KLed::On : KLed::Off);
	    break;
	default:
	    break;
	}
	return QString::null;
}
