/***************************************************************************
			kmdknob.cpp - A Knob widget for Kommander
                             -------------------
		begin			: 2005-10-01
		copyright		: (C) Pedro Lopez-Cabanillas
		email			: plcl@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* KDE INCLUDES */
#include <klocale.h>

/* QT INCLUDES */
#include <qstring.h>
#include <qwidget.h>
#include <qstringlist.h>
#include <qevent.h>

/* OTHER INCLUDES */
#include <kommanderwidget.h>
#include "dcopspecials.h"
#include "knob.h"

Knob::Knob(QWidget *a_parent, const char *a_name)
	: KnobImpl(a_parent, a_name), KommanderWidget(this)
{
	QStringList states;
	states << "default";
	setStates(states);
	setDisplayStates(states);
}

Knob::~Knob()
{
}

QString Knob::currentState() const
{
	return QString("default");
}

bool Knob::isKommanderWidget() const
{
	return TRUE;
}

QStringList Knob::associatedText() const
{
	return KommanderWidget::associatedText();
}

void Knob::setAssociatedText(QStringList a_at)
{
	KommanderWidget::setAssociatedText(a_at);
}

void Knob::populate()
{
    QString txt = KommanderWidget::evalAssociatedText( populationText() );
    setWidgetText( txt );
}

QString Knob::populationText() const
{
    return KommanderWidget::populationText();
}

void Knob::setPopulationText(QString a_text)
{
    KommanderWidget::setPopulationText(a_text);
}

void Knob::showEvent( QShowEvent *e )
{
    KnobImpl::showEvent( e );
    emit widgetOpened();
}

void Knob::setWidgetText(const QString & a_text)
{
    setValue(a_text.toInt());
    emit widgetTextChanged(a_text);
}

QString Knob::widgetText() const
{
    return QString::number(value());
}

bool Knob::isFunctionSupported(int f)
{
  return f == DCOP::text || f == DCOP::setText || f == DCOP::clear || f == DCOP::setMaximum;
}

QString Knob::handleDCOP(int function, const QStringList& args)
{
  switch (function) {
    case DCOP::text:
      return QString::number(value());
    case DCOP::setText:
      setValue(args[0].toInt());
      break;
    case DCOP::clear:
      setValue(minValue());
      break;
    case DCOP::setMaximum:
      setMaxValue(args[0].toInt());
      break;
    default:
      break;
  }
  return QString::null;
}
